@@ -48,7 +48,6 @@ typedef enum {
 	E_BOOK_ERROR_NO_SPACE
 } EBookStatus;
 
-
 typedef enum {
 	E_BOOK_VIEW_STATUS_OK,
 	E_BOOK_VIEW_STATUS_TIME_LIMIT_EXCEEDED,
@@ -71,4 +70,4 @@ typedef struct {
 
 G_END_DECLS
 
-#endif /* ! __E_BOOK_TYPES_H__ */
+#endif /* __E_BOOK_TYPES_H__ */
