@@ -27,6 +27,8 @@
 #include "e-book-backend-cache.h"
 #include "e-book-backend-sexp.h"
 
+G_DEFINE_TYPE (EBookBackendCache, e_book_backend_cache, E_TYPE_FILE_CACHE)
+
 struct _EBookBackendCachePrivate {
 	gchar *uri;
 };
@@ -189,28 +191,6 @@ e_book_backend_cache_init (EBookBackendCache *cache)
 
 }
 
-GType
-e_book_backend_cache_get_type (void)
-{
-	static GType type = 0;
-
-	if (!type) {
-		static GTypeInfo info = {
-                        sizeof (EBookBackendCacheClass),
-                        (GBaseInitFunc) NULL,
-                        (GBaseFinalizeFunc) NULL,
-                        (GClassInitFunc) e_book_backend_cache_class_init,
-                        NULL, NULL,
-                        sizeof (EBookBackendCache),
-                        0,
-                        (GInstanceInitFunc) e_book_backend_cache_init,
-                };
-		type = g_type_register_static (E_TYPE_FILE_CACHE, "EBookBackendCache", &info, 0);
-	}
-
-	return type;
-}
-
 /**
  * e_book_backend_cache_new
  * @uri: URI of the backend to be cached.
@@ -218,7 +198,7 @@ e_book_backend_cache_get_type (void)
  * Creates a new #EBookBackendCache object, which implements a local
  * cache of #EContact objects, useful for remote backends.
  *
- * Return value: A new #EBookBackendCache.
+ * Returns: A new #EBookBackendCache.
  */
 EBookBackendCache *
 e_book_backend_cache_new (const gchar *uri)
@@ -239,7 +219,7 @@ e_book_backend_cache_new (const gchar *uri)
  * newly created, and must be unreffed by the caller when no longer
  * needed.
  *
- * Return value: A cached #EContact, or %NULL if @uid is not cached.
+ * Returns: A cached #EContact, or %NULL if @uid is not cached.
  **/
 EContact *
 e_book_backend_cache_get_contact (EBookBackendCache *cache, const gchar *uid)
@@ -266,7 +246,7 @@ e_book_backend_cache_get_contact (EBookBackendCache *cache, const gchar *uid)
  *
  * Adds @contact to @cache.
  *
- * Return value: %TRUE if the contact was cached successfully, %FALSE otherwise.
+ * Returns: %TRUE if the contact was cached successfully, %FALSE otherwise.
  **/
 gboolean
 e_book_backend_cache_add_contact (EBookBackendCache *cache,
@@ -301,7 +281,7 @@ e_book_backend_cache_add_contact (EBookBackendCache *cache,
  *
  * Removes the contact identified by @uid from @cache.
  *
- * Return value: %TRUE if the contact was found and removed, %FALSE otherwise.
+ * Returns: %TRUE if the contact was found and removed, %FALSE otherwise.
  **/
 gboolean
 e_book_backend_cache_remove_contact (EBookBackendCache *cache,
@@ -332,7 +312,7 @@ e_book_backend_cache_remove_contact (EBookBackendCache *cache,
  *
  * Checks if the contact identified by @uid exists in @cache.
  *
- * Return value: %TRUE if the cache contains the contact, %FALSE otherwise.
+ * Returns: %TRUE if the cache contains the contact, %FALSE otherwise.
  **/
 gboolean
 e_book_backend_cache_check_contact (EBookBackendCache *cache, const gchar *uid)
@@ -361,7 +341,7 @@ e_book_backend_cache_check_contact (EBookBackendCache *cache, const gchar *uid)
  * When done with the list, the caller must unref the contacts and
  * free the list.
  *
- * Return value: A #GList of pointers to #EContact.
+ * Returns: A #GList of pointers to #EContact.
  **/
 GList *
 e_book_backend_cache_get_contacts (EBookBackendCache *cache, const gchar *query)
@@ -411,7 +391,7 @@ e_book_backend_cache_get_contacts (EBookBackendCache *cache, const gchar *query)
  * in @cache matching @query. When done with the array, the caller must
  * free the ID strings and the array.
  *
- * Return value: A #GPtrArray of pointers to contact ID strings.
+ * Returns: A #GPtrArray of pointers to contact ID strings.
  **/
 GPtrArray *
 e_book_backend_cache_search (EBookBackendCache *cache, const gchar *query)
@@ -438,7 +418,7 @@ e_book_backend_cache_search (EBookBackendCache *cache, const gchar *query)
  *
  * Checks if an #EBookBackendCache exists at @uri.
  *
- * Return value: %TRUE if cache exists, %FALSE if not.
+ * Returns: %TRUE if cache exists, %FALSE if not.
  **/
 gboolean
 e_book_backend_cache_exists (const gchar *uri)
@@ -476,7 +456,7 @@ e_book_backend_cache_set_populated (EBookBackendCache *cache)
  *
  * Checks if @cache is populated.
  *
- * Return value: %TRUE if @cache is populated, %FALSE otherwise.
+ * Returns: %TRUE if @cache is populated, %FALSE otherwise.
  **/
 gboolean
 e_book_backend_cache_is_populated (EBookBackendCache *cache)
