@@ -28,10 +28,11 @@
 
 #include <config.h>
 
-#include <string.h>
+#include <ctype.h>
+#include <stdio.h>
 #include <stdlib.h>
+#include <string.h>
 
-#include <glib.h>
 #include <glib/gi18n-lib.h>
 
 #include "camel-debug.h"
@@ -392,6 +393,47 @@ cdb_sql_exec (sqlite3 *db, const gchar * stmt, CamelException *ex)
 	return 0;
 }
 
+/* checks whether string 'where' contains whole word 'what',
+   case insensitively (ascii, not utf8, same as 'LIKE' in SQLite3)
+*/
+static void
+cdb_match_func (sqlite3_context *ctx, gint nArgs, sqlite3_value **values)
+{
+	gboolean matches = FALSE;
+	const gchar *what, *where;
+
+	g_return_if_fail (ctx != NULL);
+	g_return_if_fail (nArgs == 2);
+	g_return_if_fail (values != NULL);
+
+	what = (const gchar *) sqlite3_value_text (values[0]);
+	where = (const gchar *) sqlite3_value_text (values[1]);
+
+	if (what && where && !*what) {
+		matches = TRUE;
+	} else if (what && where) {
+		gboolean word = TRUE;
+		gint i, j;
+
+		for (i = 0, j = 0; where[i] && !matches; i++) {
+			gchar c = where[i];
+
+			if (c == ' ') {
+				word = TRUE;
+				j = 0;
+			} else if (word && tolower (c) == tolower(what[j])) {
+				j++;
+				if (what[j] == 0 && (where[i + 1] == 0 || isspace (where[i + 1])))
+					matches = TRUE;
+			} else {
+				word = FALSE;
+			}
+		}
+	}
+
+	sqlite3_result_int (ctx, matches ? 1 : 0);
+}
+
 /**
  * camel_db_open:
  *
@@ -432,6 +474,8 @@ camel_db_open (const gchar *path, CamelException *ex)
 	cdb->priv->timer = NULL;
 	d(g_print ("\nDatabase succesfully opened  \n"));
 
+	sqlite3_create_function (db, "MATCH", 2, SQLITE_UTF8, NULL, cdb_match_func, NULL, NULL);
+
 	/* Which is big / costlier ? A Stack frame or a pointer */
 	if (g_getenv("CAMEL_SQLITE_DEFAULT_CACHE_SIZE")!=NULL) {
 		gchar *cache = NULL;
@@ -1109,7 +1153,7 @@ camel_db_get_folder_junk_uids (CamelDB *db, gchar *folder_name, CamelException *
  * Since: 2.24
  **/
 GPtrArray *
-camel_db_get_folder_deleted_uids (CamelDB *db, gchar *folder_name, CamelException *ex)
+camel_db_get_folder_deleted_uids (CamelDB *db, const gchar *folder_name, CamelException *ex)
 {
 	 gchar *sel_query;
 	 gint ret;
@@ -1154,7 +1198,7 @@ read_preview_callback (gpointer ref, gint ncol, gchar ** cols, gchar ** name)
  * Since: 2.28
  **/
 GHashTable *
-camel_db_get_folder_preview (CamelDB *db, gchar *folder_name, CamelException *ex)
+camel_db_get_folder_preview (CamelDB *db, const gchar *folder_name, CamelException *ex)
 {
 	 gchar *sel_query;
 	 gint ret;
@@ -1179,7 +1223,7 @@ camel_db_get_folder_preview (CamelDB *db, gchar *folder_name, CamelException *ex
  * Since: 2.28
  **/
 gint
-camel_db_write_preview_record (CamelDB *db, gchar *folder_name, const gchar *uid, const gchar *msg, CamelException *ex)
+camel_db_write_preview_record (CamelDB *db, const gchar *folder_name, const gchar *uid, const gchar *msg, CamelException *ex)
 {
 	gchar *query;
 	gint ret;
@@ -1217,7 +1261,7 @@ read_vuids_callback (gpointer ref, gint ncol, gchar ** cols, gchar ** name)
  * Since: 2.24
  **/
 GPtrArray *
-camel_db_get_vuids_from_vfolder (CamelDB *db, gchar *folder_name, gchar *filter, CamelException *ex)
+camel_db_get_vuids_from_vfolder (CamelDB *db, const gchar *folder_name, gchar *filter, CamelException *ex)
 {
 	 gchar *sel_query;
 	 gchar *cond = NULL;
@@ -1910,7 +1954,7 @@ camel_db_delete_vuids (CamelDB *cdb, const gchar * folder_name, const gchar *has
  * Since: 2.24
  **/
 gint
-camel_db_clear_folder_summary (CamelDB *cdb, gchar *folder, CamelException *ex)
+camel_db_clear_folder_summary (CamelDB *cdb, const gchar *folder, CamelException *ex)
 {
 	gint ret;
 
