@@ -21,28 +21,33 @@
  *
  */
 
+#if !defined (__CAMEL_H_INSIDE__) && !defined (CAMEL_COMPILATION)
+#error "Only <camel/camel.h> can be included directly."
+#endif
+
 #ifndef CAMEL_FILTER_SEARCH_H
 #define CAMEL_FILTER_SEARCH_H
 
-#include <glib.h>
 #include <camel/camel-mime-message.h>
 #include <camel/camel-folder-summary.h>
 
 G_BEGIN_DECLS
 
+struct _CamelSession;
+
 enum {
 	CAMEL_SEARCH_ERROR    = -1,
 	CAMEL_SEARCH_NOMATCH  =  0,
 	CAMEL_SEARCH_MATCHED  =  1
 };
 
-typedef CamelMimeMessage * (*CamelFilterSearchGetMessageFunc) (void *data, CamelException *ex);
+typedef CamelMimeMessage * (*CamelFilterSearchGetMessageFunc) (gpointer data, CamelException *ex);
 
-int camel_filter_search_match (CamelSession *session,
-			       CamelFilterSearchGetMessageFunc get_message, void *data,
-			       CamelMessageInfo *info, const char *source,
-			       const char *expression, CamelException *ex);
+gint camel_filter_search_match (struct _CamelSession *session,
+			       CamelFilterSearchGetMessageFunc get_message, gpointer data,
+			       CamelMessageInfo *info, const gchar *source,
+			       const gchar *expression, CamelException *ex);
 
 G_END_DECLS
 
-#endif /* ! CAMEL_FILTER_SEARCH_H */
+#endif /* CAMEL_FILTER_SEARCH_H */
