@@ -21,8 +21,12 @@
 
 /* defines protocol for lock helper process ipc */
 
-#ifndef _CAMEL_LOCK_CLIENT_H
-#define _CAMEL_LOCK_CLIENT_H
+#if !defined (__CAMEL_H_INSIDE__) && !defined (CAMEL_COMPILATION)
+#error "Only <camel/camel.h> can be included directly."
+#endif
+
+#ifndef CAMEL_LOCK_CLIENT_H
+#define CAMEL_LOCK_CLIENT_H
 
 #include <camel/camel-exception.h>
 
@@ -33,4 +37,4 @@ gint camel_lock_helper_unlock(gint lockid);
 
 G_END_DECLS
 
-#endif /* _CAMEL_LOCK_HELPER_H */
+#endif /* CAMEL_LOCK_HELPER_H */
