@@ -20,20 +20,39 @@
  * Boston, MA 02110-1301, USA.
  */
 
-#ifndef _CAMEL_MIME_FILTER_CANON_H
-#define _CAMEL_MIME_FILTER_CANON_H
+#if !defined (__CAMEL_H_INSIDE__) && !defined (CAMEL_COMPILATION)
+#error "Only <camel/camel.h> can be included directly."
+#endif
+
+#ifndef CAMEL_MIME_FILTER_CANON_H
+#define CAMEL_MIME_FILTER_CANON_H
 
 #include <camel/camel-mime-filter.h>
 
-#define CAMEL_MIME_FILTER_CANON_TYPE         (camel_mime_filter_canon_get_type ())
-#define CAMEL_MIME_FILTER_CANON(obj)         CAMEL_CHECK_CAST (obj, CAMEL_MIME_FILTER_CANON_TYPE, CamelMimeFilterCanon)
-#define CAMEL_MIME_FILTER_CANON_CLASS(klass) CAMEL_CHECK_CLASS_CAST (klass, CAMEL_MIME_FILTER_CANON_TYPE, CamelMimeFilterCanonClass)
-#define CAMEL_IS_MIME_FILTER_CANON(obj)      CAMEL_CHECK_TYPE (obj, CAMEL_MIME_FILTER_CANON_TYPE)
+/* Standard GObject macros */
+#define CAMEL_TYPE_MIME_FILTER_CANON \
+	(camel_mime_filter_canon_get_type ())
+#define CAMEL_MIME_FILTER_CANON(obj) \
+	(G_TYPE_CHECK_INSTANCE_CAST \
+	((obj), CAMEL_TYPE_MIME_FILTER_CANON, CamelMimeFilterCanon))
+#define CAMEL_MIME_FILTER_CANON_CLASS(cls) \
+	(G_TYPE_CHECK_CLASS_CAST \
+	((cls), CAMEL_TYPE_MIME_FILTER_CANON, CamelMimeFilterCanonClass))
+#define CAMEL_IS_MIME_FILTER_CANON(obj) \
+	(G_TYPE_CHECK_INSTANCE_TYPE \
+	((obj), CAMEL_TYPE_MIME_FILTER_CANON))
+#define CAMEL_IS_MIME_FILTER_CANON_CLASS(cls) \
+	(G_TYPE_CHECK_CLASS_TYPE \
+	((cls), CAMEL_TYPE_MIME_FILTER_CANON))
+#define CAMEL_MIME_FILTER_CANON_GET_CLASS(obj) \
+	(G_TYPE_INSTANCE_GET_CLASS \
+	((obj), CAMEL_TYPE_MIME_FILTER_CANON, CamelMimeFilterCanonClass))
 
 G_BEGIN_DECLS
 
 typedef struct _CamelMimeFilterCanon CamelMimeFilterCanon;
 typedef struct _CamelMimeFilterCanonClass CamelMimeFilterCanonClass;
+typedef struct _CamelMimeFilterCanonPrivate CamelMimeFilterCanonPrivate;
 
 enum {
 	CAMEL_MIME_FILTER_CANON_CRLF = (1<<0), /* canoncialise end of line to crlf, otherwise canonicalise to lf only */
@@ -43,18 +62,17 @@ enum {
 
 struct _CamelMimeFilterCanon {
 	CamelMimeFilter parent;
-
-	guint32 flags;
+	CamelMimeFilterCanonPrivate *priv;
 };
 
 struct _CamelMimeFilterCanonClass {
 	CamelMimeFilterClass parent_class;
 };
 
-CamelType camel_mime_filter_canon_get_type (void);
-
-CamelMimeFilter *camel_mime_filter_canon_new(guint32 flags);
+GType		camel_mime_filter_canon_get_type(void);
+CamelMimeFilter *
+		camel_mime_filter_canon_new	(guint32 flags);
 
 G_END_DECLS
 
-#endif /* _CAMEL_MIME_FILTER_CANON_H */
+#endif /* CAMEL_MIME_FILTER_CANON_H */
