@@ -41,7 +41,6 @@
 #include <alloca.h>
 #endif
 
-#include <glib.h>
 #include <glib/gi18n-lib.h>
 
 #include "camel-exception.h"
@@ -86,7 +85,9 @@ static gint camel_movemail_copy(gint fromfd, gint tofd, off_t start, gsize bytes
  * Return Value: Returns -1 on error.
  **/
 gint
-camel_movemail(const gchar *source, const gchar *dest, CamelException *ex)
+camel_movemail (const gchar *source,
+                const gchar *dest,
+                CamelException *ex)
 {
 	gint lockid = -1;
 	gint res = -1;
@@ -101,11 +102,11 @@ camel_movemail(const gchar *source, const gchar *dest, CamelException *ex)
 	 * called a fraction earlier.)
 	 */
 	if (stat (source, &st) == -1) {
-		if (errno != ENOENT) {
-			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-					      _("Could not check mail file %s: %s"),
-					      source, g_strerror (errno));
-		}
+		if (errno != ENOENT)
+			camel_exception_setv (
+				ex, CAMEL_EXCEPTION_SYSTEM,
+				_("Could not check mail file %s: %s"),
+				source, g_strerror (errno));
 		return -1;
 	}
 
@@ -115,18 +116,19 @@ camel_movemail(const gchar *source, const gchar *dest, CamelException *ex)
 	/* open files */
 	sfd = open (source, O_RDWR);
 	if (sfd == -1) {
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Could not open mail file %s: %s"),
-				      source, g_strerror (errno));
+		camel_exception_setv (
+			ex, CAMEL_EXCEPTION_SYSTEM,
+			_("Could not open mail file %s: %s"),
+			source, g_strerror (errno));
 		return -1;
 	}
 
 	dfd = open (dest, O_WRONLY | O_CREAT | O_APPEND, S_IRUSR | S_IWUSR);
 	if (dfd == -1) {
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Could not open temporary mail "
-					"file %s: %s"), dest,
-				      g_strerror (errno));
+		camel_exception_setv (
+			ex, CAMEL_EXCEPTION_SYSTEM,
+			_("Could not open temporary mail "
+			"file %s: %s"), dest, g_strerror (errno));
 		close (sfd);
 		return -1;
 	}
@@ -152,9 +154,10 @@ camel_movemail(const gchar *source, const gchar *dest, CamelException *ex)
 		if (close (dfd) == 0) {
 			ftruncate (sfd, 0);
 		} else {
-			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-					      _("Failed to store mail in temp file %s: %s"),
-					      dest, g_strerror (errno));
+			camel_exception_setv (
+				ex, CAMEL_EXCEPTION_SYSTEM,
+				_("Failed to store mail in temp file %s: %s"),
+				dest, g_strerror (errno));
 			res = -1;
 		}
 	} else
@@ -168,7 +171,9 @@ camel_movemail(const gchar *source, const gchar *dest, CamelException *ex)
 
 #ifdef MOVEMAIL_PATH
 static void
-movemail_external (const gchar *source, const gchar *dest, CamelException *ex)
+movemail_external (const gchar *source,
+                   const gchar *dest,
+                   CamelException *ex)
 {
 	sigset_t mask, omask;
 	pid_t pid;
@@ -182,9 +187,10 @@ movemail_external (const gchar *source, const gchar *dest, CamelException *ex)
 
 	if (pipe (fd) == -1) {
 		sigprocmask (SIG_SETMASK, &omask, NULL);
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Could not create pipe: %s"),
-				      g_strerror (errno));
+		camel_exception_setv (
+			ex, CAMEL_EXCEPTION_SYSTEM,
+			_("Could not create pipe: %s"),
+			g_strerror (errno));
 		return;
 	}
 
@@ -194,9 +200,9 @@ movemail_external (const gchar *source, const gchar *dest, CamelException *ex)
 		close (fd[0]);
 		close (fd[1]);
 		sigprocmask (SIG_SETMASK, &omask, NULL);
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Could not fork: %s"),
-				      g_strerror (errno));
+		camel_exception_setv (
+			ex, CAMEL_EXCEPTION_SYSTEM,
+			_("Could not fork: %s"), g_strerror (errno));
 		return;
 
 	case 0:
@@ -232,9 +238,10 @@ movemail_external (const gchar *source, const gchar *dest, CamelException *ex)
 	sigprocmask (SIG_SETMASK, &omask, NULL);
 
 	if (!WIFEXITED (status) || WEXITSTATUS (status) != 0) {
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Movemail program failed: %s"),
-				      output ? output : _("(Unknown error)"));
+		camel_exception_setv (
+			ex, CAMEL_EXCEPTION_SYSTEM,
+			_("Movemail program failed: %s"),
+			output ? output : _("(Unknown error)"));
 	}
 	g_free (output);
 }
@@ -242,7 +249,9 @@ movemail_external (const gchar *source, const gchar *dest, CamelException *ex)
 
 #ifndef HAVE_BROKEN_SPOOL
 static gint
-camel_movemail_copy_file(gint sfd, gint dfd, CamelException *ex)
+camel_movemail_copy_file (gint sfd,
+                          gint dfd,
+                          CamelException *ex)
 {
 	gint nread, nwrote;
 	gchar buf[4096];
@@ -256,9 +265,10 @@ camel_movemail_copy_file(gint sfd, gint dfd, CamelException *ex)
 		else if (nread == -1) {
 			if (errno == EINTR)
 				continue;
-			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-					      _("Error reading mail file: %s"),
-					      g_strerror (errno));
+			camel_exception_setv (
+				ex, CAMEL_EXCEPTION_SYSTEM,
+				_("Error reading mail file: %s"),
+				g_strerror (errno));
 			return -1;
 		}
 
@@ -267,9 +277,10 @@ camel_movemail_copy_file(gint sfd, gint dfd, CamelException *ex)
 			if (nwrote == -1) {
 				if (errno == EINTR)
 					continue; /* continues inner loop */
-				camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-						      _("Error writing mail temp file: %s"),
-						      g_strerror (errno));
+				camel_exception_setv (
+					ex, CAMEL_EXCEPTION_SYSTEM,
+					_("Error writing mail temp file: %s"),
+					g_strerror (errno));
 				return -1;
 			}
 			written += nwrote;
@@ -447,22 +458,25 @@ solaris_header_write(gint fd, struct _camel_header_raw *header)
    we must convert it to a real mbox format.  Thankfully this is
    mostly pretty easy */
 static gint
-camel_movemail_solaris (gint oldsfd, gint dfd, CamelException *ex)
+camel_movemail_solaris (gint oldsfd,
+                        gint dfd,
+                        CamelException *ex)
 {
 	CamelMimeParser *mp;
 	gchar *buffer;
 	gint len;
 	gint sfd;
-	CamelMimeFilterFrom *ffrom;
+	CamelMimeFilter *ffrom;
 	gint ret = 1;
 	gchar *from = NULL;
 
 	/* need to dup as the mime parser will close on finish */
 	sfd = dup(oldsfd);
 	if (sfd == -1) {
-		camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				      _("Error copying mail temp file: %s"),
-				      g_strerror (errno));
+		camel_exception_setv (
+			ex, CAMEL_EXCEPTION_SYSTEM,
+			_("Error copying mail temp file: %s"),
+			g_strerror (errno));
 		return -1;
 	}
 
@@ -509,7 +523,7 @@ camel_movemail_solaris (gint oldsfd, gint dfd, CamelException *ex)
 				newpos = length+body;
 			}
 			/* copy body->length converting From lines */
-			if (camel_movemail_copy_filter(sfd, dfd, body, length, (CamelMimeFilter *)ffrom) == -1)
+			if (camel_movemail_copy_filter(sfd, dfd, body, length, ffrom) == -1)
 				goto fail;
 			if (newpos != -1)
 				camel_mime_parser_seek(mp, newpos, SEEK_SET);
@@ -519,20 +533,21 @@ camel_movemail_solaris (gint oldsfd, gint dfd, CamelException *ex)
 		g_free(from);
 	}
 
-	camel_object_unref((CamelObject *)mp);
-	camel_object_unref((CamelObject *)ffrom);
+	g_object_unref (mp);
+	g_object_unref (ffrom);
 
 	return ret;
 
 fail:
 	g_free(from);
 
-	camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-			      _("Error copying mail temp file: %s"),
-			      g_strerror (errno));
+	camel_exception_setv (
+		ex, CAMEL_EXCEPTION_SYSTEM,
+		_("Error copying mail temp file: %s"),
+		g_strerror (errno));
 
-	camel_object_unref((CamelObject *)mp);
-	camel_object_unref((CamelObject *)ffrom);
+	g_object_unref (mp);
+	g_object_unref (ffrom);
 
 	return -1;
 }
