@@ -24,13 +24,16 @@
  * USA
  */
 
+#if !defined (__CAMEL_H_INSIDE__) && !defined (CAMEL_COMPILATION)
+#error "Only <camel/camel.h> can be included directly."
+#endif
 
 #ifndef CAMEL_PROVIDER_H
-#define CAMEL_PROVIDER_H 1
+#define CAMEL_PROVIDER_H
 
-#include <camel/camel-types.h>
-#include <camel/camel-object.h>
 #include <camel/camel-exception.h>
+#include <camel/camel-object.h>
+#include <camel/camel-object-bag.h>
 #include <camel/camel-url.h>
 
 #define CAMEL_PROVIDER(obj) ((CamelProvider *)(obj))
@@ -43,7 +46,7 @@ typedef enum {
 	CAMEL_NUM_PROVIDER_TYPES
 } CamelProviderType;
 
-extern char *camel_provider_type_name[CAMEL_NUM_PROVIDER_TYPES];
+extern gchar *camel_provider_type_name[CAMEL_NUM_PROVIDER_TYPES];
 
 /* Provider flags:
  *
@@ -67,7 +70,6 @@ extern char *camel_provider_type_name[CAMEL_NUM_PROVIDER_TYPES];
 #define CAMEL_PROVIDER_HAS_LICENSE      (1 << 6)
 #define CAMEL_PROVIDER_DISABLE_SENT_FOLDER (1 << 7)
 
-
 /* Flags for url_flags. "ALLOW" means the config dialog will let the
  * user configure it. "NEED" implies "ALLOW" but means the user must
  * configure it. Service code can assume that any url part for which
@@ -117,7 +119,6 @@ extern char *camel_provider_type_name[CAMEL_NUM_PROVIDER_TYPES];
 #define CAMEL_URL_FRAGMENT_IS_PATH  (1 << 30) /* url uses fragment for folder name path, not path */
 #define CAMEL_URL_PATH_IS_ABSOLUTE (1 << 31)
 
-
 #define CAMEL_PROVIDER_IS_STORE_AND_TRANSPORT(prov) (prov->object_types[CAMEL_PROVIDER_STORE] && prov->object_types[CAMEL_PROVIDER_TRANSPORT])
 
 /* Generic extra config stuff */
@@ -135,8 +136,8 @@ typedef enum {
 
 typedef struct {
 	CamelProviderConfType type;
-	char *name, *depname;
-	char *text, *value;
+	const gchar *name, *depname;
+	const gchar *text, *value;
 } CamelProviderConfEntry;
 
 /* Some defaults */
@@ -144,96 +145,98 @@ typedef struct {
 #define CAMEL_PROVIDER_CONF_DEFAULT_HOSTNAME  { CAMEL_PROVIDER_CONF_LABEL, "hostname", NULL, N_("_Host:"), NULL }
 #define CAMEL_PROVIDER_CONF_DEFAULT_PATH      { CAMEL_PROVIDER_CONF_ENTRY, "path", NULL, N_("_Path:"), "" }
 
-typedef int (*CamelProviderAutoDetectFunc) (CamelURL *url, GHashTable **auto_detected, CamelException *ex);
+typedef gint (*CamelProviderAutoDetectFunc) (CamelURL *url, GHashTable **auto_detected, CamelException *ex);
 
 typedef struct {
 	/* Provider name used in CamelURLs. */
-	char *protocol;
-	
+	const gchar *protocol;
+
 	/* Provider name as used by people. (May be the same as protocol) */
-	char *name;
-	
+	const gchar *name;
+
 	/* Description of the provider. A novice user should be able
 	 * to read this description, and the information provided by
 	 * an ISP, IS department, etc, and determine whether or not
 	 * this provider is relevant to him, and if so, which
 	 * information goes with it.
 	 */
-	char *description;
-	
+	const gchar *description;
+
 	/* The category of message that this provider works with.
 	 * (evolution-mail will only list a provider in the store/transport
 	 * config dialogs if its domain is "mail".)
 	 */
-	char *domain;
-	
+	const gchar *domain;
+
 	/* Flags describing the provider, flags describing its URLs */
-	int flags, url_flags;
+	gint flags, url_flags;
 
 	/* The ConfEntry and AutoDetect functions will probably be
 	 * DEPRECATED in a future release */
-	
+
 	/* Extra configuration information */
 	CamelProviderConfEntry *extra_conf;
 
 	/* auto-detection function */
 	CamelProviderAutoDetectFunc auto_detect;
 
-	/* CamelType(s) of its store and/or transport. If both are
+	/* GType(s) of its store and/or transport. If both are
 	 * set, then they are assumed to be linked together and the
 	 * transport type can only be used in an account that also
 	 * uses the store type (eg, Exchange or NNTP).
 	 */
-	CamelType object_types[CAMEL_NUM_PROVIDER_TYPES];
-	
+	GType object_types[CAMEL_NUM_PROVIDER_TYPES];
+
 	/* GList of CamelServiceAuthTypes the provider supports */
 	GList *authtypes;
-	
+
 	CamelObjectBag *service_cache[CAMEL_NUM_PROVIDER_TYPES];
-	
+
 	GHashFunc url_hash;
 	GCompareFunc url_equal;
 
 	/* gettext translation domain (NULL for providers in the
 	 * evolution source tree).
 	 */
-	char *translation_domain;
+	const gchar *translation_domain;
 
 	/* This string points to the provider's gconf key value
 	 */
-	const char *license;
-	
+	const gchar *license;
+
 	/* This holds the license file name [ ascii text format ] containing
-	 * the license agreement. This should be the absolute file path. This 
+	 * the license agreement. This should be the absolute file path. This
 	 * is read only when the HAS_LICENSE flag is set
 	 */
-	const char *license_file;
+	const gchar *license_file;
 
-	/* Private to the provider */	
-	void *priv;
+	/* Private to the provider */
+	gpointer priv;
 } CamelProvider;
 
 typedef struct _CamelProviderModule CamelProviderModule;
 
 struct _CamelProviderModule {
-	char *path;
+	gchar *path;
 	GSList *types;
 	guint loaded:1;
 };
 
-void camel_provider_init(void);
-
-void camel_provider_load(const char *path, CamelException *ex);
-void camel_provider_register(CamelProvider *provider);
-GList *camel_provider_list(gboolean load);
-CamelProvider *camel_provider_get(const char *url_string, CamelException *ex);
+void		camel_provider_init		(void);
+gboolean	camel_provider_load		(const gchar *path,
+						 CamelException *ex);
+void		camel_provider_register		(CamelProvider *provider);
+GList *		camel_provider_list		(gboolean load);
+CamelProvider *	camel_provider_get		(const gchar *url_string,
+						 CamelException *ex);
 
 /* This is defined by each module, not by camel-provider.c. */
-void camel_provider_module_init(void);
+void		camel_provider_module_init	(void);
 
-
-int camel_provider_auto_detect (CamelProvider *provider, CamelURL *url,
-				GHashTable **auto_detected, CamelException *ex);
+gint		camel_provider_auto_detect	(CamelProvider *provider,
+						 CamelURL *url,
+						 GHashTable **auto_detected,
+						 CamelException *ex);
 
 G_END_DECLS
 
