@@ -23,11 +23,16 @@
  * USA
  */
 
+#if !defined (__CAMEL_H_INSIDE__) && !defined (CAMEL_COMPILATION)
+#error "Only <camel/camel.h> can be included directly."
+#endif
+
 #ifndef CAMEL_URL_H
-#define CAMEL_URL_H 1
+#define CAMEL_URL_H
 
 #include <glib.h>
-#include <camel/camel-types.h>
+
+#include <camel/camel-exception.h>
 
 G_BEGIN_DECLS
 
