@@ -209,7 +209,7 @@ maildir_append_message (CamelFolder *folder,
 	camel_local_folder_unlock (lf);
 
 	if (lf && camel_folder_change_info_changed (lf->changes)) {
-		camel_object_trigger_event (CAMEL_OBJECT (folder), "folder_changed", lf->changes);
+		camel_folder_changed (folder, lf->changes);
 		camel_folder_change_info_clear (lf->changes);
 	}
 
@@ -300,7 +300,7 @@ maildir_get_message (CamelFolder *folder,
 	camel_local_folder_unlock (lf);
 
 	if (lf && camel_folder_change_info_changed (lf->changes)) {
-		camel_object_trigger_event (CAMEL_OBJECT (folder), "folder_changed", lf->changes);
+		camel_folder_changed (folder, lf->changes);
 		camel_folder_change_info_clear (lf->changes);
 	}
 
