@@ -209,6 +209,9 @@ connect_to_server (CamelService *service, struct addrinfo *ai, gint ssl_mode, Ca
 {
 	CamelNNTPStore *store = (CamelNNTPStore *) service;
 	CamelDiscoStore *disco_store = (CamelDiscoStore*) service;
+	CamelSession *session;
+	gchar *socks_host;
+	gint socks_port;
 	CamelStream *tcp_stream;
 	gboolean retval = FALSE;
 	guchar *buf;
@@ -235,6 +238,14 @@ connect_to_server (CamelService *service, struct addrinfo *ai, gint ssl_mode, Ca
 		tcp_stream = camel_tcp_stream_raw_new ();
 	}
 
+	session = camel_service_get_session (service);
+	camel_session_get_socks_proxy (session, &socks_host, &socks_port);
+
+	if (socks_host) {
+		camel_tcp_stream_set_socks_proxy ((CamelTcpStream *) tcp_stream, socks_host, socks_port);
+		g_free (socks_host);
+	}
+
 	if (camel_tcp_stream_connect ((CamelTcpStream *) tcp_stream, ai) == -1) {
 		if (errno == EINTR)
 			camel_exception_set (
@@ -675,7 +686,7 @@ nntp_store_get_subscribed_folder_info (CamelNNTPStore *store, const gchar *top,
 					}
 					camel_service_unlock (CAMEL_SERVICE (store), CAMEL_SERVICE_REC_CONNECT_LOCK);
 					if (changes) {
-						camel_object_trigger_event((CamelObject *) folder, "folder_changed", changes);
+						camel_folder_changed (CAMEL_FOLDER (folder), changes);
 						camel_folder_change_info_free(changes);
 					}
 					g_object_unref (folder);
@@ -696,6 +707,91 @@ nntp_store_get_subscribed_folder_info (CamelNNTPStore *store, const gchar *top,
 	return first;
 }
 
+/* returns new root */
+static CamelFolderInfo *
+nntp_push_to_hierarchy (CamelURL *base_url, CamelFolderInfo *root, CamelFolderInfo *pfi, GHashTable *known)
+{
+	CamelFolderInfo *fi, *last = NULL, *kfi;
+	gchar *name, *dot;
+
+	g_return_val_if_fail (pfi != NULL, root);
+	g_return_val_if_fail (known != NULL, root);
+
+	name = pfi->full_name;
+	g_return_val_if_fail (name != NULL, root);
+
+	while (dot = strchr (name, '.'), dot) {
+		*dot = '\0';
+
+		kfi = g_hash_table_lookup (known, pfi->full_name);
+		if (!kfi) {
+			gchar *path;
+			CamelURL *url;
+
+			fi = camel_folder_info_new ();
+			fi->full_name = g_strdup (pfi->full_name);
+			fi->name = g_strdup (name);
+
+			fi->unread = 0;
+			fi->total = 0;
+			fi->flags = CAMEL_FOLDER_NOSELECT | CAMEL_FOLDER_CHILDREN;
+			path = alloca (strlen (fi->full_name) + 2);
+			sprintf (path, "/%s", fi->full_name);
+			url = camel_url_new_with_base (base_url, path);
+			fi->uri = camel_url_to_string (url, CAMEL_URL_HIDE_ALL);
+			camel_url_free (url);
+
+			g_hash_table_insert (known, fi->full_name, fi);
+			if (last) {
+				if (!last->child) {
+					last->child = fi;
+					fi->parent = last;
+				} else {
+					kfi = last->child;
+					while (kfi->next)
+						kfi = kfi->next;
+					kfi->next = fi;
+					fi->parent = last;
+				}
+			}
+			last = fi;
+			if (!root)
+				root = fi;
+		} else {
+			last = kfi;
+		}
+
+		*dot = '.';
+		name = dot + 1;
+	}
+
+	g_free (pfi->name);
+	pfi->name = g_strdup (name);
+
+	if (!root) {
+		root = pfi;
+	} else if (!last) {
+		kfi = root;
+		while (kfi->next)
+			kfi = kfi->next;
+		kfi->next = pfi;
+		pfi->parent = kfi->parent;
+	} else {
+		if (!last->child) {
+			last->child = pfi;
+			pfi->parent = last;
+		} else {
+			kfi = last->child;
+			while (kfi->next)
+				kfi = kfi->next;
+			kfi->next = pfi;
+			pfi->parent = last;
+		}
+	}
+
+	return root;
+}
+
 /*
  * get folder info, using the information in our StoreSummary
  */
@@ -705,17 +801,21 @@ nntp_store_get_cached_folder_info (CamelNNTPStore *store, const gchar *orig_top,
 	gint i;
 	gint subscribed_or_flag = (flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIBED) ? 0 : 1,
 	    root_or_flag = (orig_top == NULL || orig_top[0] == '\0') ? 1 : 0,
-	    recursive_flag = flags & CAMEL_STORE_FOLDER_INFO_RECURSIVE;
+	    recursive_flag = flags & CAMEL_STORE_FOLDER_INFO_RECURSIVE,
+	    is_folder_list = flags & CAMEL_STORE_FOLDER_INFO_SUBSCRIPTION_LIST;
 	CamelStoreInfo *si;
-	CamelFolderInfo *first = NULL, *last = NULL, *fi = NULL, *l;
+	CamelFolderInfo *first = NULL, *last = NULL, *fi = NULL;
+	GHashTable *known; /* folder name to folder info */
 	gchar *tmpname;
 	gchar *top = g_strconcat(orig_top?orig_top:"", ".", NULL);
 	gint toplen = strlen(top);
 
+	known = g_hash_table_new (g_str_hash, g_str_equal);
+
 	for (i = 0; (si = camel_store_summary_index ((CamelStoreSummary *) store->summary, i)); i++) {
 		if ((subscribed_or_flag || (si->flags & CAMEL_STORE_INFO_FOLDER_SUBSCRIBED))
 		    && (root_or_flag || strncmp (si->path, top, toplen) == 0)) {
-			if (recursive_flag || strchr (si->path + toplen, '.') == NULL) {
+			if (recursive_flag || is_folder_list || strchr (si->path + toplen, '.') == NULL) {
 				/* add the item */
 				fi = nntp_folder_info_from_store_info(store, FALSE, si);
 				if (!fi)
@@ -748,24 +848,24 @@ nntp_store_get_cached_folder_info (CamelNNTPStore *store, const gchar *orig_top,
 				}
 			}
 
-			for (l = first; l; l = l->next) {
-				if (l->full_name && fi->full_name && g_str_equal (l->full_name, fi->full_name)) {
-					/* it's there already, do not add it twice */
-					camel_folder_info_free (fi);
-					break;
-				}
-			}
-
-			if (l) {
+			if (fi->full_name && g_hash_table_lookup (known, fi->full_name)) {
 				/* a duplicate has been found above */
+				camel_folder_info_free (fi);
 				continue;
 			}
 
-			if (last)
-				last->next = fi;
-			else
-				first = fi;
-			last = fi;
+			g_hash_table_insert (known, fi->full_name, fi);
+
+			if (is_folder_list) {
+				/* create a folder hierarchy rather than a flat list */
+				first = nntp_push_to_hierarchy (((CamelService *)store)->url, first, fi, known);
+			} else {
+				if (last)
+					last->next = fi;
+				else
+					first = fi;
+				last = fi;
+			}
 		} else if (subscribed_or_flag && first) {
 			/* we have already added subitems, but this item is no longer a subitem */
 			camel_store_summary_info_free((CamelStoreSummary *)store->summary, si);
@@ -774,6 +874,7 @@ nntp_store_get_cached_folder_info (CamelNNTPStore *store, const gchar *orig_top,
 		camel_store_summary_info_free((CamelStoreSummary *)store->summary, si);
 	}
 
+	g_hash_table_destroy (known);
 	g_free(top);
 	return first;
 }
@@ -936,7 +1037,7 @@ nntp_get_folder_info_offline(CamelStore *store, const gchar *top, guint32 flags,
 }
 
 static gboolean
-nntp_store_folder_subscribed (CamelStore *store, const gchar *folder_name)
+nntp_store_folder_is_subscribed (CamelStore *store, const gchar *folder_name)
 {
 	CamelNNTPStore *nntp_store = CAMEL_NNTP_STORE (store);
 	CamelStoreInfo *si;
@@ -978,7 +1079,7 @@ nntp_store_subscribe_folder (CamelStore *store, const gchar *folder_name,
 			camel_store_summary_touch ((CamelStoreSummary *) nntp_store->summary);
 			camel_store_summary_save ((CamelStoreSummary *) nntp_store->summary);
 			camel_service_unlock (CAMEL_SERVICE (nntp_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
-			camel_object_trigger_event ((CamelObject *) nntp_store, "folder_subscribed", fi);
+			camel_store_folder_subscribed (CAMEL_STORE (nntp_store), fi);
 			camel_folder_info_free (fi);
 			return TRUE;
 		}
@@ -1015,7 +1116,7 @@ nntp_store_unsubscribe_folder (CamelStore *store, const gchar *folder_name,
 			camel_store_summary_touch ((CamelStoreSummary *) nntp_store->summary);
 			camel_store_summary_save ((CamelStoreSummary *) nntp_store->summary);
 			camel_service_unlock (CAMEL_SERVICE (nntp_store), CAMEL_SERVICE_REC_CONNECT_LOCK);
-			camel_object_trigger_event ((CamelObject *) nntp_store, "folder_unsubscribed", fi);
+			camel_store_folder_unsubscribed (CAMEL_STORE (nntp_store), fi);
 			camel_folder_info_free (fi);
 			return TRUE;
 		}
@@ -1149,7 +1250,7 @@ camel_nntp_store_class_init (CamelNNTPStoreClass *class)
 
 	store_class = CAMEL_STORE_CLASS (class);
 	store_class->free_folder_info = camel_store_free_folder_info_full;
-	store_class->folder_subscribed = nntp_store_folder_subscribed;
+	store_class->folder_is_subscribed = nntp_store_folder_is_subscribed;
 	store_class->subscribe_folder = nntp_store_subscribe_folder;
 	store_class->unsubscribe_folder = nntp_store_unsubscribe_folder;
 	store_class->create_folder = nntp_create_folder;
@@ -1217,12 +1318,14 @@ camel_nntp_try_authenticate (CamelNNTPStore *store, CamelException *ex)
 
 		service->url->passwd =
 			camel_session_get_password (session, service, NULL,
-						    prompt, "password", CAMEL_SESSION_PASSWORD_SECRET, ex);
+						    prompt, "password", CAMEL_SESSION_PASSWORD_SECRET | (store->password_reprompt ? CAMEL_SESSION_PASSWORD_REPROMPT : 0), ex);
 		g_free(prompt);
 		g_free(base);
 
 		if (!service->url->passwd)
 			return -1;
+
+		store->password_reprompt = FALSE;
 	}
 
 	/* now, send auth info (currently, only authinfo user/pass is supported) */
@@ -1236,8 +1339,8 @@ camel_nntp_try_authenticate (CamelNNTPStore *store, CamelException *ex)
 			    camel_exception_get_id (ex) == CAMEL_EXCEPTION_SERVICE_UNAVAILABLE)
 				return ret;
 
-			/* Need to forget the password here since we have no context on it */
-			camel_session_forget_password(session, service, NULL, "password", ex);
+			/* To force password reprompt */
+			store->password_reprompt = TRUE;
 			g_free (service->url->passwd);
 			service->url->passwd = NULL;
 			goto retry;
